@@ -7,6 +7,8 @@ class BackEndTypes:
     Tensorflow2 = 'TF2'
     Pytorch = 'PYT'
 
+    ALL = [Sklearn, Tensorflow1, Tensorflow2, Pytorch]
+
 
 class SamplingStrategy:
     Random = 'random'
@@ -14,6 +16,13 @@ class SamplingStrategy:
     KdTree = 'kdtree'
 
 
+class ModelTypes:
+    Classifier = 'classifier'
+    Regressor = 'regressor'
+
+    ALL = [Classifier, Regressor]
+
+
 class _SchemaVersions:
     V1 = '1.0'
     V2 = '2.0'
