@@ -6,10 +6,11 @@ import numpy as np
 import logging
 from collections import defaultdict
 
-from dice_ml.utils.exception import UserConfigValidationException
+from dice_ml.data_interfaces.base_data_interface import _BaseData
+from dice_ml.utils.exception import SystemException, UserConfigValidationException
 
 
-class PublicData:
+class PublicData(_BaseData):
     """A data interface for public data. This class is an interface to DiCE explainers
        and contains methods to transform user-fed raw data into the format a DiCE explainer
        requires, and vice versa."""
@@ -24,31 +25,10 @@ class PublicData:
                                            Defaults to the range inferred from training data.
         :param continuous_features_precision (optional): Dictionary with feature names as keys and precisions as values.
         :param data_name (optional): Dataset name
-
         """
-
-        if isinstance(params['dataframe'], pd.DataFrame):
-            self.data_df = params['dataframe']
-        else:
-            raise ValueError("should provide a pandas dataframe")
-
-        if type(params['continuous_features']) is list:
-            self.continuous_feature_names = params['continuous_features']
-        else:
-            raise ValueError(
-                "should provide the name(s) of continuous features in the data as a list")
-
-        if type(params['outcome_name']) is str:
-            self.outcome_name = params['outcome_name']
-        else:
-            raise ValueError("should provide the name of outcome feature as a string")
-
-        if params['outcome_name'] not in self.data_df.columns.tolist():
-            raise UserConfigValidationException(
-                "outcome_name {0} not found in {1}".format(
-                    params['outcome_name'], ','.join(self.data_df.columns.tolist())
-                )
-            )
+        self._validate_and_set_outcome_name(params=params)
+        self._validate_and_set_dataframe(params=params)
+        self._validate_and_set_continuous_features(params=params)
 
         self.feature_names = [
             name for name in self.data_df.columns.tolist() if name != self.outcome_name]
@@ -63,21 +43,10 @@ class PublicData:
         self.categorical_feature_names = [name for name in self.data_df.columns.tolist(
         ) if name not in self.continuous_feature_names + [self.outcome_name]]
 
-        self.continuous_feature_indexes = [self.data_df.columns.get_loc(
-            name) for name in self.continuous_feature_names if name in self.data_df]
-
         self.categorical_feature_indexes = [self.data_df.columns.get_loc(
             name) for name in self.categorical_feature_names if name in self.data_df]
 
-        if 'continuous_features_precision' in params:
-            self.continuous_features_precision = params['continuous_features_precision']
-            for continuous_features_precision_feature_name in self.continuous_features_precision:
-                if continuous_features_precision_feature_name not in self.feature_names:
-                    raise UserConfigValidationException(
-                        "continuous_features_precision contains some feature names which are not part of columns in dataframe"
-                    )
-        else:
-            self.continuous_features_precision = None
+        self._validate_and_set_continuous_features_precision(params=params)
 
         if len(self.categorical_feature_names) > 0:
             for feature in self.categorical_feature_names:
@@ -109,25 +78,105 @@ class PublicData:
         #     self.labelencoder[column] = LabelEncoder()
         #     self.label_encoded_data[column] = self.labelencoder[column].fit_transform(self.data_df[column])
 
-        input_permitted_range = None
-        if 'permitted_range' in params:
-            input_permitted_range = params['permitted_range']
-            for input_permitted_range_feature_name in input_permitted_range:
-                if input_permitted_range_feature_name not in self.feature_names:
-                    raise UserConfigValidationException(
-                        "permitted_range contains some feature names which are not part of columns in dataframe"
-                    )
-        self.permitted_range, feature_ranges_orig = self.get_features_range(input_permitted_range)
+        self._validate_and_set_permitted_range(params=params)
 
         # should move the below snippet to model agnostic dice interfaces
         # self.max_range = -np.inf
         # for feature in self.continuous_feature_names:
         #     self.max_range = max(self.max_range, self.permitted_range[feature][1])
 
-        if 'data_name' in params:
-            self.data_name = params['data_name']
+        self._validate_and_set_data_name(params=params)
+
+    def _validate_and_set_dataframe(self, params):
+        """Validate and set the dataframe."""
+        if 'dataframe' not in params:
+            raise ValueError("dataframe not found in params")
+
+        if isinstance(params['dataframe'], pd.DataFrame):
+            self.data_df = params['dataframe'].copy()
         else:
-            self.data_name = 'mydata'
+            raise ValueError("should provide a pandas dataframe")
+
+        if 'outcome_name' in params and params['outcome_name'] not in self.data_df.columns.tolist():
+            raise UserConfigValidationException(
+                "outcome_name {0} not found in {1}".format(
+                    params['outcome_name'], ','.join(self.data_df.columns.tolist())
+                )
+            )
+
+    def _validate_and_set_continuous_features(self, params):
+        """Validate and set the list of continuous features."""
+        if 'continuous_features' not in params:
+            raise ValueError('continuous_features should be provided')
+
+        if type(params['continuous_features']) is list:
+            self.continuous_feature_names = params['continuous_features']
+        else:
+            raise ValueError(
+                "should provide the name(s) of continuous features in the data as a list")
+
+    def _validate_and_set_continuous_features_precision(self, params):
+        """Validate and set the dictionary of precision for continuous features."""
+        if 'continuous_features_precision' in params:
+            self.continuous_features_precision = params['continuous_features_precision']
+
+            if not hasattr(self, 'feature_names'):
+                raise SystemException('Feature names not correctly set in public data interface')
+
+            for continuous_features_precision_feature_name in self.continuous_features_precision:
+                if continuous_features_precision_feature_name not in self.feature_names:
+                    raise UserConfigValidationException(
+                        "continuous_features_precision contains some feature names which are not part of columns in dataframe"
+                    )
+        else:
+            self.continuous_features_precision = None
+
+    def _validate_and_set_permitted_range(self, params):
+        """Validate and set the dictionary of permitted ranges for continuous features."""
+        input_permitted_range = None
+        if 'permitted_range' in params:
+            input_permitted_range = params['permitted_range']
+
+            if not hasattr(self, 'feature_names'):
+                raise SystemException('Feature names not correctly set in public data interface')
+
+            for input_permitted_range_feature_name in input_permitted_range:
+                if input_permitted_range_feature_name not in self.feature_names:
+                    raise UserConfigValidationException(
+                        "permitted_range contains some feature names which are not part of columns in dataframe"
+                    )
+        self.permitted_range, _ = self.get_features_range(input_permitted_range)
+
+    def check_features_to_vary(self, features_to_vary):
+        if features_to_vary is not None and features_to_vary != 'all':
+            not_training_features = set(features_to_vary) - set(self.feature_names)
+            if len(not_training_features) > 0:
+                raise UserConfigValidationException("Got features {0} which are not present in training data".format(
+                    not_training_features))
+
+    def check_permitted_range(self, permitted_range):
+        if permitted_range is not None:
+            permitted_range_features = list(permitted_range)
+            not_training_features = set(permitted_range_features) - set(self.feature_names)
+            if len(not_training_features) > 0:
+                raise UserConfigValidationException("Got features {0} which are not present in training data".format(
+                    not_training_features))
+
+            for feature in permitted_range_features:
+                if feature in self.categorical_feature_names:
+                    train_categories = self.permitted_range[feature]
+                    for test_category in permitted_range[feature]:
+                        if test_category not in train_categories:
+                            raise UserConfigValidationException(
+                                'The category {0} does not occur in the training data for feature {1}.'
+                                ' Allowed categories are {2}'.format(test_category, feature, train_categories))
+
+    def check_mad_validity(self, feature_weights):
+        """checks feature MAD validity and throw warnings.
+           TODO: add comments as to where this is used if this function is necessary, else remove.
+        """
+        if feature_weights == "inverse_mad":
+            self.get_valid_mads(display_warnings=True, return_mads=False)
 
     def get_features_range(self, permitted_range_input=None):
         ranges = {}
@@ -146,9 +195,9 @@ class PublicData:
 
     def get_data_type(self, col):
         """Infers data type of a continuous feature from the training data."""
-        if ((self.data_df[col].dtype == np.int64) or (self.data_df[col].dtype == np.int32)):
+        if (self.data_df[col].dtype == np.int64) or (self.data_df[col].dtype == np.int32):
             return 'int'
-        elif ((self.data_df[col].dtype == np.float64) or (self.data_df[col].dtype == np.float32)):
+        elif (self.data_df[col].dtype == np.float64) or (self.data_df[col].dtype == np.float32):
             return 'float'
         else:
             raise ValueError("Unknown data type of feature %s: must be int or float" % col)
@@ -164,7 +213,10 @@ class PublicData:
             for feature_name in self.continuous_feature_names:
                 max_value = self.data_df[feature_name].max()
                 min_value = self.data_df[feature_name].min()
-                result[feature_name] = (df[feature_name] - min_value) / (max_value - min_value)
+                if min_value == max_value:
+                    result[feature_name] = 0
+                else:
+                    result[feature_name] = (df[feature_name] - min_value) / (max_value - min_value)
         else:
             result = result.astype('float')
             for feature_index in self.continuous_feature_indexes:
@@ -172,10 +224,16 @@ class PublicData:
                 max_value = self.data_df[feature_name].max()
                 min_value = self.data_df[feature_name].min()
                 if len(df.shape) == 1:
-                    value = (df[feature_index] - min_value) / (max_value - min_value)
+                    if min_value == max_value:
+                        value = 0
+                    else:
+                        value = (df[feature_index] - min_value) / (max_value - min_value)
                     result[feature_index] = value
                 else:
-                    result[:, feature_index] = (df[:, feature_index] - min_value) / (max_value - min_value)
+                    if min_value == max_value:
+                        result[:, feature_index] = np.zeros(len(df[:, feature_index]))
+                    else:
+                        result[:, feature_index] = (df[:, feature_index] - min_value) / (max_value - min_value)
         return result
 
     def de_normalize_data(self, df):
@@ -404,10 +462,10 @@ class PublicData:
         precisions_dict = defaultdict(int)
         precisions = [0] * len(self.feature_names)
         for ix, col in enumerate(self.continuous_feature_names):
-            if ((self.continuous_features_precision is not None) and (col in self.continuous_features_precision)):
+            if (self.continuous_features_precision is not None) and (col in self.continuous_features_precision):
                 precisions[ix] = self.continuous_features_precision[col]
                 precisions_dict[col] = self.continuous_features_precision[col]
-            elif ((self.data_df[col].dtype == np.float32) or (self.data_df[col].dtype == np.float64)):
+            elif self.data_df[col].dtype == np.float32 or self.data_df[col].dtype == np.float64:
                 modes = self.data_df[col].mode()
                 maxp = len(str(modes[0]).split('.')[1])  # maxp stores the maximum precision of the modes
                 for mx in range(len(modes)):
