@@ -2,6 +2,7 @@
    Subclasses implement interfaces for different ML frameworks such as TensorFlow or PyTorch.
    All methods are in dice_ml.explainer_interfaces"""
 
+from abc import ABC, abstractmethod
 import numpy as np
 import pandas as pd
 from tqdm import tqdm
@@ -10,9 +11,10 @@ from collections.abc import Iterable
 from sklearn.neighbors import KDTree
 from dice_ml.counterfactual_explanations import CounterfactualExplanations
 from dice_ml.utils.exception import UserConfigValidationException
+from dice_ml.constants import ModelTypes
 
 
-class ExplainerBase:
+class ExplainerBase(ABC):
 
     def __init__(self, data_interface, model_interface=None):
         """Init method
@@ -53,7 +55,6 @@ class ExplainerBase:
         :param query_instances: Input point(s) for which counterfactuals are to be generated.
                                 This can be a dataframe with one or more rows.
         :param total_CFs: Total number of counterfactuals required.
-
         :param desired_class: Desired counterfactual class - can take 0 or 1. Default value
                               is "opposite" to the outcome class of query_instance for binary classification.
         :param desired_range: For regression problems. Contains the outcome range to
@@ -74,7 +75,7 @@ class ExplainerBase:
         :param kwargs: Other parameters accepted by specific explanation method
 
         :returns: A CounterfactualExplanations object that contains the list of
-        counterfactual examples per query_instance as one of its attributes.
+                  counterfactual examples per query_instance as one of its attributes.
         """
         if total_CFs <= 0:
             raise UserConfigValidationException(
@@ -87,6 +88,7 @@ class ExplainerBase:
         elif isinstance(query_instances, Iterable):
             query_instances_list = query_instances
         for query_instance in tqdm(query_instances_list):
+            self.data_interface.set_continuous_feature_indexes(query_instance)
             res = self._generate_counterfactuals(
                 query_instance, total_CFs,
                 desired_class=desired_class,
@@ -101,7 +103,46 @@ class ExplainerBase:
             cf_examples_arr.append(res)
         return CounterfactualExplanations(cf_examples_list=cf_examples_arr)
 
+    @abstractmethod
+    def _generate_counterfactuals(self, query_instance, total_CFs,
+                                  desired_class="opposite", desired_range=None,
+                                  permitted_range=None, features_to_vary="all",
+                                  stopping_threshold=0.5, posthoc_sparsity_param=0.1,
+                                  posthoc_sparsity_algorithm="linear", verbose=False, **kwargs):
+        """Internal method for generating counterfactuals for a given query instance. Any explainerclass
+           inherting from this class would need to implement this abstract method.
+
+        :param query_instance: Input point for which counterfactuals are to be generated.
+                               This can be a dataframe with one row.
+        :param total_CFs: Total number of counterfactuals required.
+        :param desired_class: Desired counterfactual class - can take 0 or 1. Default value
+                              is "opposite" to the outcome class of query_instance for binary classification.
+        :param desired_range: For regression problems. Contains the outcome range to
+                              generate counterfactuals in.
+        :param permitted_range: Dictionary with feature names as keys and permitted range in list as values.
+                                Defaults to the range inferred from training data.
+                                If None, uses the parameters initialized in data_interface.
+        :param features_to_vary: Either a string "all" or a list of feature names to vary.
+        :param stopping_threshold: Minimum threshold for counterfactuals target class probability.
+        :param posthoc_sparsity_param: Parameter for the post-hoc operation on continuous features to enhance sparsity.
+        :param posthoc_sparsity_algorithm: Perform either linear or binary search. Takes "linear" or "binary".
+                                           Prefer binary search when a feature range is large (for instance,
+                                           income varying from 10k to 1000k) and only if the features share a
+                                           monotonic relationship with predicted outcome in the model.
+        :param verbose: Whether to output detailed messages.
+        :param sample_size: Sampling size
+        :param random_seed: Random seed for reproducibility
+        :param kwargs: Other parameters accepted by specific explanation method
+
+        :returns: A CounterfactualExplanations object that contains the list of
+                  counterfactual examples per query_instance as one of its attributes.
+        """
+        pass
+
     def setup(self, features_to_vary, permitted_range, query_instance, feature_weights):
+        self.data_interface.check_features_to_vary(features_to_vary=features_to_vary)
+        self.data_interface.check_permitted_range(permitted_range)
+
         if features_to_vary == 'all':
             features_to_vary = self.data_interface.feature_names
 
@@ -110,14 +151,22 @@ class ExplainerBase:
             feature_ranges_orig = self.feature_range
         else:  # compute the new ranges based on user input
             self.feature_range, feature_ranges_orig = self.data_interface.get_features_range(permitted_range)
+
         self.check_query_instance_validity(features_to_vary, permitted_range, query_instance, feature_ranges_orig)
 
         # check feature MAD validity and throw warnings
-        self.check_mad_validity(feature_weights)
+        self.data_interface.check_mad_validity(feature_weights)
 
         return features_to_vary
 
     def check_query_instance_validity(self, features_to_vary, permitted_range, query_instance, feature_ranges_orig):
+        for feature in query_instance:
+            if feature == self.data_interface.outcome_name:
+                raise ValueError("Target", self.data_interface.outcome_name, "present in query instance")
+
+            if feature not in self.data_interface.feature_names:
+                raise ValueError("Feature", feature, "not present in training data!")
+
         for feature in self.data_interface.categorical_feature_names:
             if query_instance[feature].values[0] not in feature_ranges_orig[feature]:
                 raise ValueError("Feature", feature, "has a value outside the dataset.")
@@ -199,7 +248,7 @@ class ExplainerBase:
                   the list of counterfactuals per input, local feature importances per
                   input, and the global feature importance summarized over all inputs.
         """
-        if len(query_instances) < 10:
+        if query_instances is not None and len(query_instances) < 10:
             raise UserConfigValidationException("The number of query instances should be greater than or equal to 10")
         if cf_examples_list is not None:
             if any([len(cf_examples.final_cfs_df) < 10 for cf_examples in cf_examples_list]):
@@ -273,6 +322,7 @@ class ExplainerBase:
                 for col in allcols:
                     local_importances[i][col] = 0
 
+        overall_num_cfs = 0
         # Summarizing the found counterfactuals
         for i in range(len(cf_examples_list)):
             cf_examples = cf_examples_list[i]
@@ -282,7 +332,13 @@ class ExplainerBase:
                 df = cf_examples.final_cfs_df_sparse
             else:
                 df = cf_examples.final_cfs_df
+
+            if df is None:
+                continue
+
+            per_query_point_cfs = 0
             for index, row in df.iterrows():
+                per_query_point_cfs += 1
                 for col in self.data_interface.continuous_feature_names:
                     if not np.isclose(org_instance[col].iat[0], row[col]):
                         if summary_importance is not None:
@@ -298,15 +354,25 @@ class ExplainerBase:
 
             if local_importances is not None:
                 for col in allcols:
-                    local_importances[i][col] /= (cf_examples_list[0].final_cfs_df.shape[0])
+                    if per_query_point_cfs > 0:
+                        local_importances[i][col] /= per_query_point_cfs
+
+            overall_num_cfs += per_query_point_cfs
+
         if summary_importance is not None:
             for col in allcols:
-                summary_importance[col] /= (cf_examples_list[0].final_cfs_df.shape[0]*len(cf_examples_list))
+                if overall_num_cfs > 0:
+                    summary_importance[col] /= overall_num_cfs
+
         return CounterfactualExplanations(
             cf_examples_list,
             local_importance=local_importances,
             summary_importance=summary_importance)
 
+    def predict_fn(self, input_instance):
+        """prediction function"""
+        return self.model.get_output(input_instance)
+
     def predict_fn_for_sparsity(self, input_instance):
         """prediction function for sparsity correction"""
         return self.model.get_output(input_instance)
@@ -327,8 +393,6 @@ class ExplainerBase:
         if final_cfs_sparse is None:
             return final_cfs_sparse
 
-        # resetting index to make sure .loc works
-        final_cfs_sparse = final_cfs_sparse.reset_index(drop=True)
         # quantiles of the deviation from median for every continuous feature
         quantiles = self.data_interface.get_quantiles_from_training_data(quantile=posthoc_sparsity_param)
         mads = self.data_interface.get_valid_mads()
@@ -347,13 +411,12 @@ class ExplainerBase:
 
         cfs_preds_sparse = []
 
-        for cf_ix in range(len(final_cfs_sparse)):
-            current_pred = self.predict_fn_for_sparsity(
-                final_cfs_sparse.iloc[[cf_ix]][self.data_interface.feature_names])
+        for cf_ix in list(final_cfs_sparse.index):
+            current_pred = self.predict_fn_for_sparsity(final_cfs_sparse.loc[[cf_ix]][self.data_interface.feature_names])
             for feature in features_sorted:
                 # current_pred = self.predict_fn_for_sparsity(final_cfs_sparse.iat[[cf_ix]][self.data_interface.feature_names])
                 # feat_ix = self.data_interface.continuous_feature_names.index(feature)
-                diff = query_instance[feature].iat[0] - final_cfs_sparse[feature].iat[cf_ix]
+                diff = query_instance[feature].iat[0] - int(final_cfs_sparse.at[cf_ix, feature])
                 if(abs(diff) <= quantiles[feature]):
                     if posthoc_sparsity_algorithm == "linear":
                         final_cfs_sparse = self.do_linear_search(diff, decimal_prec, query_instance, cf_ix,
@@ -363,7 +426,7 @@ class ExplainerBase:
                         final_cfs_sparse = self.do_binary_search(
                             diff, decimal_prec, query_instance, cf_ix, feature, final_cfs_sparse, current_pred)
 
-            temp_preds = self.predict_fn_for_sparsity(final_cfs_sparse.iloc[[cf_ix]][self.data_interface.feature_names])
+            temp_preds = self.predict_fn_for_sparsity(final_cfs_sparse.loc[[cf_ix]][self.data_interface.feature_names])
             cfs_preds_sparse.append(temp_preds)
 
         final_cfs_sparse[self.data_interface.outcome_name] = self.get_model_output_from_scores(cfs_preds_sparse)
@@ -377,19 +440,19 @@ class ExplainerBase:
         old_diff = diff
         change = (10**-decimal_prec[feature])  # the minimal possible change for a feature
         current_pred = current_pred_orig
-        if self.model.model_type == 'classifier':
+        if self.model.model_type == ModelTypes.Classifier:
             while((abs(diff) > 10e-4) and (np.sign(diff*old_diff) > 0) and self.is_cf_valid(current_pred)):
-                old_val = final_cfs_sparse[feature].iat[cf_ix]
-                final_cfs_sparse.loc[cf_ix, feature] += np.sign(diff)*change
-                current_pred = self.predict_fn_for_sparsity(final_cfs_sparse.iloc[[cf_ix]][self.data_interface.feature_names])
+                old_val = int(final_cfs_sparse.at[cf_ix, feature])
+                final_cfs_sparse.at[cf_ix, feature] += np.sign(diff)*change
+                current_pred = self.predict_fn_for_sparsity(final_cfs_sparse.loc[[cf_ix]][self.data_interface.feature_names])
                 old_diff = diff
 
                 if not self.is_cf_valid(current_pred):
-                    final_cfs_sparse.loc[cf_ix, feature] = old_val
-                    diff = query_instance[feature].iat[0] - final_cfs_sparse[feature].iat[cf_ix]
+                    final_cfs_sparse.at[cf_ix, feature] = old_val
+                    diff = query_instance[feature].iat[0] - int(final_cfs_sparse.at[cf_ix, feature])
                     return final_cfs_sparse
 
-                diff = query_instance[feature].iat[0] - final_cfs_sparse[feature].iat[cf_ix]
+                diff = query_instance[feature].iat[0] - int(final_cfs_sparse.at[cf_ix, feature])
 
         return final_cfs_sparse
 
@@ -397,28 +460,28 @@ class ExplainerBase:
         """Performs a binary search between continuous features of a CF and corresponding values
            in query_instance until the prediction class changes."""
 
-        old_val = final_cfs_sparse[feature].iat[cf_ix]
-        final_cfs_sparse.loc[cf_ix, feature] = query_instance[feature].iat[0]
+        old_val = int(final_cfs_sparse.at[cf_ix, feature])
+        final_cfs_sparse.at[cf_ix, feature] = query_instance[feature].iat[0]
         # Prediction of the query instance
-        current_pred = self.predict_fn_for_sparsity(final_cfs_sparse.iloc[[cf_ix]][self.data_interface.feature_names])
+        current_pred = self.predict_fn_for_sparsity(final_cfs_sparse.loc[[cf_ix]][self.data_interface.feature_names])
 
         # first check if assigning query_instance values to a CF is required.
         if self.is_cf_valid(current_pred):
             return final_cfs_sparse
         else:
-            final_cfs_sparse.loc[cf_ix, feature] = old_val
+            final_cfs_sparse.at[cf_ix, feature] = old_val
 
         # move the CF values towards the query_instance
         if diff > 0:
-            left = final_cfs_sparse[feature].iat[cf_ix]
+            left = int(final_cfs_sparse.at[cf_ix, feature])
             right = query_instance[feature].iat[0]
 
             while left <= right:
                 current_val = left + ((right - left)/2)
                 current_val = round(current_val, decimal_prec[feature])
 
-                final_cfs_sparse.loc[cf_ix, feature] = current_val
-                current_pred = self.predict_fn_for_sparsity(final_cfs_sparse.iloc[[cf_ix]][self.data_interface.feature_names])
+                final_cfs_sparse.at[cf_ix, feature] = current_val
+                current_pred = self.predict_fn_for_sparsity(final_cfs_sparse.loc[[cf_ix]][self.data_interface.feature_names])
 
                 if current_val == right or current_val == left:
                     break
@@ -430,14 +493,14 @@ class ExplainerBase:
 
         else:
             left = query_instance[feature].iat[0]
-            right = final_cfs_sparse[feature].iat[cf_ix]
+            right = int(final_cfs_sparse.at[cf_ix, feature])
 
             while right >= left:
                 current_val = right - ((right - left)/2)
                 current_val = round(current_val, decimal_prec[feature])
 
-                final_cfs_sparse.loc[cf_ix, feature] = current_val
-                current_pred = self.predict_fn_for_sparsity(final_cfs_sparse.iloc[[cf_ix]][self.data_interface.feature_names])
+                final_cfs_sparse.at[cf_ix, feature] = current_val
+                current_pred = self.predict_fn_for_sparsity(final_cfs_sparse.loc[[cf_ix]][self.data_interface.feature_names])
 
                 if current_val == right or current_val == left:
                     break
@@ -451,7 +514,7 @@ class ExplainerBase:
 
     def misc_init(self, stopping_threshold, desired_class, desired_range, test_pred):
         self.stopping_threshold = stopping_threshold
-        if self.model.model_type == 'classifier':
+        if self.model.model_type == ModelTypes.Classifier:
             self.target_cf_class = np.array(
                 [[self.infer_target_cfs_class(desired_class, test_pred, self.num_output_nodes)]],
                 dtype=np.float32)
@@ -461,15 +524,15 @@ class ExplainerBase:
             elif self.target_cf_class == 1 and self.stopping_threshold < 0.5:
                 self.stopping_threshold = 0.75
 
-        elif self.model.model_type == 'regressor':
+        elif self.model.model_type == ModelTypes.Regressor:
             self.target_cf_range = self.infer_target_cfs_range(desired_range)
         return desired_class
 
     def infer_target_cfs_class(self, desired_class_input, original_pred, num_output_nodes):
         """ Infer the target class for generating CFs. Only called when
             model_type=="classifier".
-            TODO: Add support for opposite desired class in multiclass. Downstream methods should decide
-                  whether it is allowed or not.
+            TODO: Add support for opposite desired class in multiclass.
+            Downstream methods should decide whether it is allowed or not.
         """
         if desired_class_input == "opposite":
             if num_output_nodes == 2:
@@ -504,7 +567,7 @@ class ExplainerBase:
         validity = np.zeros(len(model_outputs), dtype=np.int32)
         for i in range(len(model_outputs)):
             pred = model_outputs[i]
-            if self.model.model_type == "classifier":
+            if self.model.model_type == ModelTypes.Classifier:
                 if self.num_output_nodes == 2:  # binary
                     pred_1 = pred[self.num_output_nodes-1]
                     validity[i] = 1 if \
@@ -513,7 +576,7 @@ class ExplainerBase:
                 else:  # multiclass
                     if np.argmax(pred) == self.target_cf_class:
                         validity[i] = 1
-            elif self.model.model_type == "regressor":
+            elif self.model.model_type == ModelTypes.Regressor:
                 if self.target_cf_range[0] <= pred <= self.target_cf_range[1]:
                     validity[i] = 1
         return validity
@@ -523,11 +586,11 @@ class ExplainerBase:
         """
         # Converting to single prediction if the prediction is provided as a
         # singleton array
-        correct_dim = 1 if self.model.model_type == "classifier" else 0
+        correct_dim = 1 if self.model.model_type == ModelTypes.Classifier else 0
         if hasattr(model_score, "shape") and len(model_score.shape) > correct_dim:
             model_score = model_score[0]
         # Converting target_cf_class to a scalar (tf/torch have it as (1,1) shape)
-        if self.model.model_type == 'classifier':
+        if self.model.model_type == ModelTypes.Classifier:
             target_cf_class = self.target_cf_class
             if hasattr(self.target_cf_class, "shape"):
                 if len(self.target_cf_class.shape) == 1:
@@ -549,24 +612,20 @@ class ExplainerBase:
                      (target_cf_class == 1 and pred_1 >= self.stopping_threshold)) else False
                 return validity
             else:  # multiclass
-                if np.argmax(model_score) == target_cf_class:
-                    return True
-        elif self.model.model_type == "regressor":
-            if self.target_cf_range[0] <= model_score <= self.target_cf_range[1]:
-                return True
-            else:
-                return False
+                return np.argmax(model_score) == target_cf_class
+        else:
+            return self.target_cf_range[0] <= model_score and model_score <= self.target_cf_range[1]
 
     def get_model_output_from_scores(self, model_scores):
-        if self.model.model_type == "classifier":
+        if self.model.model_type == ModelTypes.Classifier:
             output_type = np.int32
         else:
             output_type = np.float32
         model_output = np.zeros(len(model_scores), dtype=output_type)
         for i in range(len(model_scores)):
-            if self.model.model_type == "classifier":
+            if self.model.model_type == ModelTypes.Classifier:
                 model_output[i] = np.argmax(model_scores[i])
-            elif self.model.model_type == "regressor":
+            elif self.model.model_type == ModelTypes.Regressor:
                 model_output[i] = model_scores[i]
         return model_output
 
@@ -587,13 +646,6 @@ class ExplainerBase:
                 self.cont_minx.append(self.data_interface.permitted_range[feature][0])
                 self.cont_maxx.append(self.data_interface.permitted_range[feature][1])
 
-    def check_mad_validity(self, feature_weights):
-        """checks feature MAD validity and throw warnings.
-           TODO: add comments as to where this is used if this function is necessary, else remove.
-        """
-        if feature_weights == "inverse_mad":
-            self.data_interface.get_valid_mads(display_warnings=True, return_mads=False)
-
     def sigmoid(self, z):
         """This is used in VAE-based CF explainers."""
         return 1 / (1 + np.exp(-z))
@@ -609,10 +661,10 @@ class ExplainerBase:
 
         # segmenting the dataset according to outcome
         dataset_with_predictions = None
-        if self.model.model_type == 'classifier':
+        if self.model.model_type == ModelTypes.Classifier:
             dataset_with_predictions = data_df_copy.loc[[i == desired_class for i in predictions]].copy()
 
-        elif self.model.model_type == 'regressor':
+        elif self.model.model_type == ModelTypes.Regressor:
             dataset_with_predictions = data_df_copy.loc[
                 [desired_range[0] <= pred <= desired_range[1] for pred in predictions]].copy()
 
