@@ -1,6 +1,8 @@
 import pytest
 from collections import OrderedDict
 import pandas as pd
+from sklearn.datasets import load_iris, load_boston
+from sklearn.model_selection import train_test_split
 import dice_ml
 from dice_ml.utils import helpers
 
@@ -16,6 +18,17 @@ def binary_classification_exp_object(method="random"):
     return exp
 
 
+@pytest.fixture
+def binary_classification_exp_object_out_of_order(method="random"):
+    backend = 'sklearn'
+    dataset = helpers.load_outcome_not_last_column_dataset()
+    d = dice_ml.Data(dataframe=dataset, continuous_features=['Numerical'], outcome_name='Outcome')
+    ML_modelpath = helpers.get_custom_dataset_modelpath_pipeline_binary()
+    m = dice_ml.Model(model_path=ML_modelpath, backend=backend)
+    exp = dice_ml.Dice(d, m, method=method)
+    return exp
+
+
 @pytest.fixture
 def multi_classification_exp_object(method="random"):
     backend = 'sklearn'
@@ -108,6 +121,30 @@ def sample_custom_query_4():
     return pd.DataFrame({'Categorical': ['c'], 'Numerical': [13]})
 
 
+@pytest.fixture
+def sample_custom_query_5():
+    """
+    Returns a sample query instance for the custom dataset
+    """
+    return pd.DataFrame({'X': ['d'], 'Numerical': [25]})
+
+
+@pytest.fixture
+def sample_custom_query_6():
+    """
+    Returns a sample query instance for the custom dataset including Outcome
+    """
+    return pd.DataFrame({'Categorical': ['c'], 'Numerical': [13], 'Outcome': 0})
+
+
+@pytest.fixture
+def sample_custom_query_index():
+    """
+    Returns a sample query instance for the custom dataset
+    """
+    return pd.DataFrame({'Categorical': ['a'], 'Numerical': [88]})
+
+
 @pytest.fixture
 def sample_custom_query_10():
     """
@@ -115,7 +152,46 @@ def sample_custom_query_10():
     """
     return pd.DataFrame(
         {
-            'Categorical': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j'],
+            'Categorical': ['a', 'b', 'c', 'a', 'b', 'c', 'a', 'b', 'c', 'a'],
             'Numerical': [25, 50, 75, 100, 125, 150, 175, 200, 225, 250]
         }
     )
+
+
+@pytest.fixture
+def sample_counterfactual_example_dummy():
+    """
+    Returns a sample counterfactual example
+    """
+    return pd.DataFrame(
+        {
+            'Categorical': ['a', 'b', 'c', 'a', 'b',
+                            'c', 'a', 'b', 'c', 'a',
+                            'a', 'b', 'c', 'c', 'c'],
+            'Numerical': [25, 50, 75, 100, 125,
+                          150, 175, 200, 225, 250,
+                          150, 175, 200, 225, 250],
+            'Outcome': [1, 1, 1, 1, 1,
+                        1, 1, 1, 1, 1,
+                        1, 1, 1, 1, 1]
+        }
+    )
+
+
+@pytest.fixture
+def create_iris_data():
+    iris = load_iris()
+    x_train, x_test, y_train, y_test = train_test_split(
+        iris.data, iris.target, test_size=0.2, random_state=0)
+    feature_names = iris.feature_names
+    classes = iris.target_names
+    return x_train, x_test, y_train, y_test, feature_names, classes
+
+
+@pytest.fixture
+def create_boston_data():
+    boston = load_boston()
+    x_train, x_test, y_train, y_test = train_test_split(
+        boston.data, boston.target,
+        test_size=0.2, random_state=7)
+    return x_train, x_test, y_train, y_test, boston.feature_names
